/* NiceXSL to XSL translator main program

 Copyright (c) 2002 The Regents of the University of California.
 All rights reserved.
 Permission is hereby granted, without written agreement and without
 license or royalty fees, to use, copy, modify, and distribute this
 software and its documentation for any purpose, provided that the above
 copyright notice and the following two paragraphs appear in all copies
 of this software.

 IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
 THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
 SUCH DAMAGE.

 THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
 CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
 ENHANCEMENTS, OR MODIFICATIONS.

                                        PT_COPYRIGHT_VERSION_2
                                        COPYRIGHTENDKEY

@ProposedRating Red (Ed.Willink@uk.thalesgroup.com)
@AcceptedRating Red

Created : August 2002

Version 1.2: 27-Aug-2002
Version 1.2.1: 3-Sep-2002 Use upper case class names

*/
package nicexsl;

import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Document;
import javax.xml.parsers.DocumentBuilderFactory;
import org.apache.xalan.serialize.Serializer;
import org.apache.xalan.serialize.SerializerFactory;
import java.util.Properties;
import java.io.ByteArrayOutputStream;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.OutputStream;
import java.io.PrintWriter;

///////////main//////////////////////////////////////////////////////////////
//// main

/**
The NiceXSL grammar defines a conventional textual representation of XSL that
is more amenable to convential editing.
<p>
The forward transformation from NiceXSL to XSL is provided by this example main program in
the nicexsl.main class. It may be invoked in the standard distribution as:
<br><tt>java -jar nicexsl_1v2.jar <em>input-file.nicexsl</em> <em>output-file.xsl</em></tt>
<p>
The main program is a simple driver that constructs a nicexsl.lexer to read from a file stream
and invokes the parse() function of nicexsl.parser to populate a DOM Document, which is then
converted to an XML string for output as XSL. Custom applications may therefore embed the
NiceXSL parser by copying about 10 lines from the example main program. 
<p>
The reverse transformation from XSL to NiceXSL is provided by <tt>XSL2NiceXSL.xsl</tt>. This may
be used to convert existing code, or to get examples of the NiceXSL form of the construct.

@author Ed Willink
@version $Id$
@see nicexsl.Lexer nicexsl.Parser
*/
public class Main {

    //
    //  Main program parsing args[0] as NiceXSL to DOM and then writing to args[1] as XSL.
    //
    public static void main(String args[]) throws Exception {
        if (args.length != 2) {
            printUsage();
            return;
        }
        FileInputStream inputStream = new FileInputStream(args[0]);
        Lexer aLexer = new Lexer(inputStream);
        Parser aParser = new Parser(aLexer);
        DOMImplementation domImpl = DocumentBuilderFactory.newInstance().newDocumentBuilder().getDOMImplementation();
        Document aDocument = domImpl.createDocument("", "xsl:stylesheet", null);
        aDocument.insertBefore(aDocument.createComment(" Generated by NiceXSLParser version 1.2 "), aDocument.getDocumentElement());
        try {
            aParser.parse(aDocument, args[0], false);
            String result = createXML(aDocument);
            OutputStream os = null;
            boolean closeStream = false;
            if (".".equals(args[1])) {
                os = System.out;
            } else {
                os = new FileOutputStream(args[1]);
                closeStream = true;
            }
            PrintWriter pw = new PrintWriter(os);
            pw.print(result);
            if (closeStream)
                pw.close();
            else
                pw.flush();
            return;
        }
        catch (Exception e) {
            throw new Exception(e.toString());  // Throw a new exception to lose the back trace.
        }
    }

    //
    //  Return the 'doc' as an XML string.
    //
    private static String createXML(Document doc) throws Exception {
        Properties format = org.apache.xalan.templates.OutputProperties.getDefaultMethodProperties("xml");
        format.setProperty("indent", "yes");
        format.setProperty("{http://xml.apache.org/xslt}indent-amount", "4");
        Serializer ser = SerializerFactory.getSerializer(format);
        OutputStream os = new ByteArrayOutputStream();
        ser.setOutputStream(os);
        ser.asDOMSerializer().serialize(doc);
        os.close();
        return os.toString();
    }

    //
    //  Print the program usage..
    //
    private static void printUsage() {
        System.out.println("nicexsl.main <infile> <outfile> (\".\" is standard out)");
    }


//    private static String defaultDBFI = System.getProperty("javax.xml.parsers.DocumentBuilderFactory");

//    public  static void setDefaultDBFI() {
//        System.setProperty("javax.xml.parsers.DocumentBuilderFactory", defaultDBFI == null ? "" : defaultDBFI);
//    }
}

