<!-- XSL to NiceXSL translator

 Copyright (c) 2002 The Regents of the University of California.
 All rights reserved.
 Permission is hereby granted, without written agreement and without
 license or royalty fees, to use, copy, modify, and distribute this
 software and its documentation for any purpose, provided that the above
 copyright notice and the following two paragraphs appear in all copies
 of this software.

 IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
 THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
 SUCH DAMAGE.

 THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
 CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
 ENHANCEMENTS, OR MODIFICATIONS.

                                        PT_COPYRIGHT_VERSION_2
                                        COPYRIGHTENDKEY

@ProposedRating Red (Ed.Willink@uk.thalesgroup.com)
@AcceptedRating Red

Created : June 2002

Revised:  6-Aug-2002
    Eliminate ; following {} and use sufficient {} to compensate.
    Always use () around parameters
    Let missing with-param value stay missing.
    Handle parameters on apply-templates and match
Revised:  8-Aug-2002
    Fix new line end of file..
    Get blank line in front of template comments
Revised: 11-Aug-2002
    Add XSL 2.0 constructs
    Allow multi-line strings to support XML properly.
    Preserve text/element node ordering.
    Start text on new line
Version 1.1: 13-Aug-2002
Revised: 22-Aug-2002
    Use parentheses around XPath and other expressions
Revised: 27-Aug-2002
    Add missing xsl:function translation
    Normalize unquoted spaces
    Remove redundant quotes on names
    xsl:transform too
Version 1.2: 27-Aug-2002

See main.java for the NiceXSL language definition,
and NiceXSL.cup for the inverse translation.
-->
<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  version="1.0">
    <xsl:output method="text"/>
    <xsl:strip-space elements="*"/>

    <!-- 
        Write out the current attribute as '@name="expr"'.
    -->
    <xsl:template name="write-attribute">
        <xsl:value-of select="' @'"/>
        <xsl:value-of select="name()"/>
        <xsl:value-of select="' ='"/>
        <xsl:call-template name="write-quoted">
            <xsl:with-param name="name" select="."/>
        </xsl:call-template>
    </xsl:template>

    <!-- 
        Write out all the attributes, putting out the 'quoted1'  and'quoted2' attribute first
        formatting 'quoted1' and 'quoted2' with a prefix '='. Any 'except1' and 'except2'
        attributes are omitted..
    -->
    <xsl:template name="write-attributes">
        <xsl:param name="name1" select="''"/>
        <xsl:param name="quoted1" select="''"/>
        <xsl:param name="quoted2" select="''"/>
        <xsl:param name="xpath1" select="''"/>
        <xsl:param name="xpath2" select="''"/>
        <xsl:param name="except1" select="''"/>
        <xsl:param name="except2" select="''"/>
        <xsl:for-each select="@*[name()=$name1]">
            <xsl:call-template name="write-name">
                <xsl:with-param name="name" select="."/>
            </xsl:call-template>
        </xsl:for-each>
        <xsl:for-each select="@*[name()=$quoted1]">
            <xsl:call-template name="write-quoted">
                <xsl:with-param name="name" select="."/>
            </xsl:call-template>
        </xsl:for-each>
        <xsl:for-each select="@*[name()=$xpath1]">
            <xsl:call-template name="write-xpath">
                <xsl:with-param name="name" select="."/>
            </xsl:call-template>
        </xsl:for-each>
        <xsl:for-each select="@*[name()=$quoted2]">
            <xsl:value-of select="' ='"/>
            <xsl:call-template name="write-quoted">
                <xsl:with-param name="name" select="."/>
            </xsl:call-template>
        </xsl:for-each>
        <xsl:for-each select="@*[name()=$xpath2]">
            <xsl:value-of select="' ='"/>
            <xsl:call-template name="write-xpath">
                <xsl:with-param name="name" select="."/>
            </xsl:call-template>
        </xsl:for-each>
        <xsl:for-each select="@*[name()!=$name1 and name()!=$quoted1 and name()!=$quoted2
                and name()!=$xpath1 and name()!=$xpath2 and name()!=$except1 and name()!=$except2]">
            <xsl:call-template name="write-attribute"/>
        </xsl:for-each>
    </xsl:template>

    <!-- 
        Write out indentation to reflect the prevailing hierarchy.
    -->
    <xsl:template name="write-indentation">
        <xsl:value-of select="'&#10;'"/>
        <xsl:for-each select="ancestor::*">
            <xsl:value-of select="'    '"/>
        </xsl:for-each>
    </xsl:template>
 
    <!-- 
        Write the keyword 'name' at the start of an xsl construct on a new-line with indentation
    -->
    <xsl:template name="write-keyword">
        <xsl:param name="name" select="local-name()"/>
        <xsl:call-template name="write-indentation"/>
        <xsl:value-of select="$name"/>
    </xsl:template>

    <xsl:template name="write-name">
        <xsl:param name="name"/>
        <xsl:param name="prefix" select="' '"/>
        <xsl:value-of select="$prefix"/>
        <xsl:choose>
            <xsl:when test="contains($name, '{')">
                <xsl:value-of select="'&quot;'"/>
                <xsl:call-template name="write-text">
                    <xsl:with-param name="name" select="$name"/>
                </xsl:call-template>
                <xsl:value-of select="'&quot;'"/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="$name"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <!-- 
        Write out the current namespace attribute with indentation.
    -->
    <xsl:template name="write-namespace">
        <xsl:call-template name="write-indentation"/>
        <xsl:value-of select="'xmlns:'"/>
        <xsl:value-of select="name()"/>
        <xsl:value-of select="' ='"/>
        <xsl:call-template name="write-quoted">
            <xsl:with-param name="name" select="."/>
        </xsl:call-template>
    </xsl:template>
        
    <!-- 
        Write out all the nodes, omitting 'except'.
    -->
    <xsl:template name="write-nodes">
        <xsl:param name="except" select="''"/>
        <xsl:param name="no-punct" select="false()"/>
        <xsl:choose>
            <xsl:when test="node()[name()!=$except]">
                <xsl:if test="not($no-punct)">
                    <xsl:value-of select="' {'"/>
                </xsl:if>
                <xsl:apply-templates select="text()|node()[name()!=$except]"/>
                <xsl:if test="not($no-punct)">
                    <xsl:call-template name="write-keyword">
                        <xsl:with-param name="name" select="'}'"/>
                    </xsl:call-template>
                </xsl:if>
            </xsl:when>
            <xsl:otherwise>
                <xsl:apply-templates select="text()|node()[name()!=$except]"/>
                <xsl:if test="not($no-punct)">
                    <xsl:value-of select="';'"/>
                </xsl:if>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <!-- 
        Write out the 'parameters' in parentheses.
    -->
    <xsl:template name="write-parameters">
        <xsl:param name="parameters"/>
        <xsl:value-of select="'('"/>
        <xsl:for-each select="$parameters">
            <xsl:call-template name="write-name">
                <xsl:with-param name="name" select="@name"/>
                <xsl:with-param name="prefix"/>
            </xsl:call-template>
            <xsl:choose>
                <xsl:when test="@select">
                    <xsl:value-of select="' := '"/>
                    <xsl:call-template name="write-text">
                        <xsl:with-param name="name" select="@select"/>
                    </xsl:call-template>
                </xsl:when>
                <xsl:when test="*">
                    <xsl:value-of select="' := {'"/>
                    <xsl:call-template name="write-nodes">
                        <xsl:with-param name="no-punct" select="true()"/>
                    </xsl:call-template>
                    <xsl:if test="node() | text()">
                        <xsl:value-of select="' '"/>
                    </xsl:if>
                    <xsl:value-of select="'}'"/>
                </xsl:when>
            </xsl:choose>
            <xsl:call-template name="write-attributes">
                <xsl:with-param name="except1" select="'name'"/>
                <xsl:with-param name="except2" select="'select'"/>
            </xsl:call-template>
            <xsl:if test="position() != last()">
                <xsl:value-of select="', '"/>
            </xsl:if>
        </xsl:for-each>
        <xsl:value-of select="')'"/>
    </xsl:template>
 
    <!--
        Write out the '.' attribute as name="value".
    -->
    <xsl:template name="write-quoted">
        <xsl:param name="name"/>
        <xsl:value-of select="' &quot;'"/>
        <xsl:call-template name="write-text">
            <xsl:with-param name="name" select="$name"/>
        </xsl:call-template>
        <xsl:value-of select="'&quot;'"/>
    </xsl:template>

    <!--
        Write out 'name' as text avoiding nasty characters.
    -->
    <xsl:template name="write-text">
        <xsl:param name="name"/>
        <xsl:param name="normalize-space" select="true()"/>
        <xsl:variable name="single-quote">'</xsl:variable>
        <xsl:choose>
            <xsl:when test="contains($name, '&amp;')">
                <xsl:call-template name="write-text">
                    <xsl:with-param name="name" select="substring-before($name, '&amp;')"/>
                    <xsl:with-param name="normalize-space" select="$normalize-space"/>
                </xsl:call-template>
                <xsl:value-of select="'&amp;amp;'"/>
                <xsl:call-template name="write-text">
                    <xsl:with-param name="name" select="substring-after($name, '&amp;')"/>
                    <xsl:with-param name="normalize-space" select="$normalize-space"/>
                </xsl:call-template>
            </xsl:when>
            <xsl:when test="contains($name, '&quot;') and not(contains(substring-before($name, '&quot;'), $single-quote))">
                <xsl:call-template name="write-text">
                    <xsl:with-param name="name" select="substring-before($name, '&quot;')"/>
                    <xsl:with-param name="normalize-space" select="$normalize-space"/>
                </xsl:call-template>
                <xsl:value-of select="'&amp;quot;'"/>
                <xsl:variable name="suffix" select="substring-after($name, '&quot;')"/>
                <xsl:choose>
                    <xsl:when test="contains($suffix, '&quot;')">
                        <xsl:call-template name="write-text">
                            <xsl:with-param name="name" select="substring-before($suffix, '&quot;')"/>
                            <xsl:with-param name="normalize-space" select="false()"/>
                        </xsl:call-template>
                        <xsl:value-of select="'&amp;quot;'"/>
                        <xsl:call-template name="write-text">
                            <xsl:with-param name="name" select="substring-after($suffix, '&quot;')"/>
                            <xsl:with-param name="normalize-space" select="$normalize-space"/>
                        </xsl:call-template>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:call-template name="write-text">
                            <xsl:with-param name="name" select="$suffix"/>
                            <xsl:with-param name="normalize-space" select="$normalize-space"/>
                        </xsl:call-template>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:when test="contains($name, $single-quote)">
                <xsl:call-template name="write-text">
                    <xsl:with-param name="name" select="substring-before($name, $single-quote)"/>
                    <xsl:with-param name="normalize-space" select="$normalize-space"/>
                </xsl:call-template>
                <xsl:value-of select="$single-quote"/>
                <xsl:variable name="suffix" select="substring-after($name, $single-quote)"/>
                <xsl:call-template name="write-text">
                    <xsl:with-param name="name" select="substring-before($suffix, $single-quote)"/>
                    <xsl:with-param name="normalize-space" select="false()"/>
                </xsl:call-template>
                <xsl:choose>
                    <xsl:when test="contains($suffix, $single-quote)">
                        <xsl:value-of select="$single-quote"/>
                        <xsl:call-template name="write-text">
                            <xsl:with-param name="name" select="substring-after($suffix, $single-quote)"/>
                            <xsl:with-param name="normalize-space" select="$normalize-space"/>
                        </xsl:call-template>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:call-template name="write-text">
                            <xsl:with-param name="name" select="$suffix"/>
                            <xsl:with-param name="normalize-space" select="$normalize-space"/>
                        </xsl:call-template>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:when test="contains($name, '&#10;')">
                <xsl:call-template name="write-text">
                    <xsl:with-param name="name" select="substring-before($name, '&#10;')"/>
                    <xsl:with-param name="normalize-space" select="$normalize-space"/>
                </xsl:call-template>
                <xsl:value-of select="'&amp;#xA;'"/>
                <xsl:call-template name="write-text">
                    <xsl:with-param name="name" select="substring-after($name, '&#10;')"/>
                    <xsl:with-param name="normalize-space" select="$normalize-space"/>
                </xsl:call-template>
            </xsl:when>
            <xsl:when test="contains($name, '&#13;')">
                <xsl:call-template name="write-text">
                    <xsl:with-param name="name" select="substring-before($name, '&#13;')"/>
                    <xsl:with-param name="normalize-space" select="$normalize-space"/>
                </xsl:call-template>
                <xsl:value-of select="'&amp;#xD;'"/>
                <xsl:call-template name="write-text">
                    <xsl:with-param name="name" select="substring-after($name, '&#13;')"/>
                    <xsl:with-param name="normalize-space" select="$normalize-space"/>
                </xsl:call-template>
            </xsl:when>
            <xsl:otherwise>
                <xsl:choose>
                    <xsl:when test="$normalize-space">
                        <xsl:value-of select="normalize-space($name)"/>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:value-of select="$name"/>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <xsl:template name="write-xpath">
        <xsl:param name="name"/>
        <xsl:value-of select="' ('"/>
        <xsl:call-template name="write-text">
            <xsl:with-param name="name" select="$name"/>
        </xsl:call-template>
        <xsl:value-of select="')'"/>
    </xsl:template>

    <!--
        Constructs with no or only optional attributes.
    -->
    <xsl:template match="xsl:apply-imports | xsl:choose | xsl:comment | xsl:copy
                       | xsl:decimal-format | xsl:element | xsl:fallback | xsl:import-schema
                       | xsl:message | xsl:number | xsl:otherwise | xsl:principal-result-document
                       | xsl:result-document | xsl:text">
        <xsl:call-template name="write-keyword"/>
        <xsl:call-template name="write-attributes"/>
        <xsl:call-template name="write-nodes"/>
    </xsl:template>

    <xsl:template match="xsl:apply-templates">
        <xsl:call-template name="write-keyword"/>
        <xsl:call-template name="write-xpath">
            <xsl:with-param name="name" select="@select"/>
        </xsl:call-template>
        <xsl:if test="xsl:with-param">
            <xsl:call-template name="write-parameters">
                <xsl:with-param name="parameters" select="xsl:with-param"/>
            </xsl:call-template>
        </xsl:if>
        <xsl:call-template name="write-attributes">
            <xsl:with-param name="except1" select="'select'"/>
        </xsl:call-template>
        <xsl:call-template name="write-nodes">
            <xsl:with-param name="except" select="'xsl:with-param'"/>
        </xsl:call-template>
    </xsl:template>

    <!--
        Constructs with a name attribute.
    -->
    <xsl:template match="xsl:attribute | xsl:attribute-set | xsl:key | xsl:namespace
                       | xsl:processing-instruction | xsl:sort-key">
        <xsl:call-template name="write-keyword"/>
        <xsl:call-template name="write-attributes">
            <xsl:with-param name="name1" select="'name'"/>
        </xsl:call-template>
        <xsl:call-template name="write-nodes"/>
    </xsl:template>

    <xsl:template match="xsl:call-template">
        <xsl:call-template name="write-keyword">
            <xsl:with-param name="name" select="'call'"/>
        </xsl:call-template>
        <xsl:call-template name="write-name">
            <xsl:with-param name="name" select="@name"/>
        </xsl:call-template>
        <xsl:call-template name="write-parameters">
            <xsl:with-param name="parameters" select="xsl:with-param"/>
        </xsl:call-template>
        <xsl:call-template name="write-attributes">
            <xsl:with-param name="except1" select="'name'"/>
        </xsl:call-template>
        <xsl:call-template name="write-nodes">
            <xsl:with-param name="except" select="'xsl:with-param'"/>
        </xsl:call-template>
    </xsl:template>

    <!--
        Constructs with a select attribute.
    -->
    <xsl:template match="xsl:copy-of | xsl:for-each | xsl:for-each-group | xsl:result | xsl:sort | xsl:value-of">
        <xsl:call-template name="write-keyword"/>
        <xsl:call-template name="write-attributes">
            <xsl:with-param name="xpath1" select="'select'"/>
        </xsl:call-template>
        <xsl:call-template name="write-nodes"/>
    </xsl:template>

    <xsl:template match="xsl:function">
        <xsl:if test="preceding-sibling::node()[1][node()]">
            <xsl:value-of select="'&#10;'"/>
        </xsl:if>
        <xsl:call-template name="write-keyword">
            <xsl:with-param name="name" select="'function'"/>
        </xsl:call-template>
        <xsl:call-template name="write-name">
            <xsl:with-param name="name" select="@name"/>
        </xsl:call-template>
        <xsl:call-template name="write-parameters">
            <xsl:with-param name="parameters" select="xsl:param"/>
        </xsl:call-template>
        <xsl:call-template name="write-attributes">
            <xsl:with-param name="except1" select="'name'"/>
        </xsl:call-template>
        <xsl:call-template name="write-nodes">
            <xsl:with-param name="except" select="'xsl:param'"/>
        </xsl:call-template>
    </xsl:template>

    <!--
        Constructs with a test attribute.
    -->
    <xsl:template match="xsl:if | xsl:when">
        <xsl:call-template name="write-keyword"/>
        <xsl:call-template name="write-attributes">
            <xsl:with-param name="xpath1" select="'test'"/>
        </xsl:call-template>
        <xsl:call-template name="write-nodes"/>
    </xsl:template>

    <!--
        Constructs with a href attribute.
    -->
    <xsl:template match="xsl:import | xsl:include">
        <xsl:call-template name="write-keyword"/>
        <xsl:call-template name="write-attributes">
            <xsl:with-param name="text" select="'href'"/>
        </xsl:call-template>
        <xsl:call-template name="write-nodes"/>
    </xsl:template>

    <xsl:template match="xsl:namespace-alias">
        <xsl:call-template name="write-keyword"/>
        <xsl:call-template name="write-attributes">
            <xsl:with-param name="quoted1" select="'stylesheet-prefix'"/>
            <xsl:with-param name="quoted2" select="'result-prefix'"/>
        </xsl:call-template>
        <xsl:call-template name="write-nodes"/>
    </xsl:template>

    <!--
        Constructs with a method attribute.
    -->
    <xsl:template match="xsl:output">
        <xsl:call-template name="write-keyword"/>
        <xsl:call-template name="write-attributes">
            <xsl:with-param name="name1" select="'method'"/>
        </xsl:call-template>
        <xsl:call-template name="write-nodes"/>
    </xsl:template>

    <!--
        Constructs with an elements attribute.
    -->
    <xsl:template match="xsl:preserve-space | xsl:strip-space">
        <xsl:call-template name="write-keyword"/>
        <xsl:call-template name="write-attributes">
            <xsl:with-param name="name1" select="'elements'"/>
        </xsl:call-template>
        <xsl:call-template name="write-nodes"/>
    </xsl:template>

    <xsl:template match="xsl:stylesheet | xsl:transform">
        <xsl:call-template name="write-keyword"/>
        <xsl:if test="xsl:param">
            <xsl:call-template name="write-parameters">
                <xsl:with-param name="parameters" select="xsl:param"/>
            </xsl:call-template>
        </xsl:if>
        <xsl:call-template name="write-attributes"/>
        <xsl:for-each select="namespace::*">
            <xsl:call-template name="write-namespace"/>
        </xsl:for-each>
        <xsl:call-template name="write-nodes">
            <xsl:with-param name="except" select="'xsl:param'"/>
        </xsl:call-template>
    </xsl:template>

    <xsl:template match="xsl:template">
        <xsl:if test="preceding-sibling::node()[1][node()]">
            <xsl:value-of select="'&#10;'"/>
        </xsl:if>
        <xsl:choose>
            <xsl:when test="@match">
                <xsl:call-template name="write-keyword">
                    <xsl:with-param name="name" select="'match'"/>
                </xsl:call-template>
                <xsl:call-template name="write-xpath">
                    <xsl:with-param name="name" select="@match"/>
                </xsl:call-template>
                <xsl:if test="xsl:param">
                    <xsl:call-template name="write-parameters">
                        <xsl:with-param name="parameters" select="xsl:param"/>
                    </xsl:call-template>
                </xsl:if>
                <xsl:call-template name="write-attributes">
                    <xsl:with-param name="except1" select="'match'"/>
                </xsl:call-template>
                <xsl:call-template name="write-nodes">
                    <xsl:with-param name="except" select="'xsl:param'"/>
                </xsl:call-template>
            </xsl:when>
            <xsl:otherwise>
                <xsl:call-template name="write-keyword">
                    <xsl:with-param name="name" select="'function'"/>
                </xsl:call-template>
                <xsl:call-template name="write-name">
                    <xsl:with-param name="name" select="@name"/>
                </xsl:call-template>
                <xsl:call-template name="write-parameters">
                    <xsl:with-param name="parameters" select="xsl:param"/>
                </xsl:call-template>
                <xsl:call-template name="write-attributes">
                    <xsl:with-param name="except1" select="'name'"/>
                </xsl:call-template>
                <xsl:call-template name="write-nodes">
                    <xsl:with-param name="except" select="'xsl:param'"/>
                </xsl:call-template>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <xsl:template match="xsl:variable">
        <xsl:call-template name="write-keyword"/>
        <xsl:call-template name="write-attributes">
            <xsl:with-param name="name1" select="'name'"/>
            <xsl:with-param name="xpath2" select="'select'"/>
        </xsl:call-template>
        <xsl:call-template name="write-nodes"/>
    </xsl:template>

    <xsl:template match="/">
        <xsl:value-of select="'&lt;!--'"/>
        <xsl:value-of select="' Generated by XSL2NiceXSL.xsl version 1.2 '"/>
        <xsl:value-of select="'--&gt;'"/>
        <xsl:value-of select="'&#10;'"/>
        <xsl:apply-templates select="@*"/>
        <xsl:apply-templates select="text()|node()"/>
        <xsl:value-of select="'&#10;'"/>
    </xsl:template>

    <xsl:template match="text()">
        <xsl:choose>
            <xsl:when test="not(..[self::xsl:text])">
                <xsl:call-template name="write-keyword">
                    <xsl:with-param name="name" select="'&quot;'"/>
                </xsl:call-template>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="' &quot;'"/>
            </xsl:otherwise>
        </xsl:choose>
        <xsl:call-template name="write-text">
            <xsl:with-param name="name" select="."/>
            <xsl:with-param name="normalize-space" select="false()"/>
        </xsl:call-template>
        <xsl:value-of select="'&quot;'"/>
    </xsl:template>

    <xsl:template match="comment()">
        <xsl:if test="following-sibling::node()[1][self::xsl:template or self::xsl:function]">
            <xsl:value-of select="'&#10;'"/>
        </xsl:if>
        <xsl:call-template name="write-keyword"/>
        <xsl:value-of select="'&lt;!--'"/>
        <xsl:value-of select="."/>
        <xsl:value-of select="'--&gt;'"/>
    </xsl:template>

    <xsl:template match="@*">
        <xsl:call-template name="write-attribute"/>
    </xsl:template>

    <xsl:template match="xsl:*">
        <xsl:message>No match for construct.<xsl:value-of select="name()"/></xsl:message>
        <xsl:call-template name="write-keyword">
            <xsl:with-param name="name" select="concat('&lt;', name())"/>
        </xsl:call-template>
        <xsl:call-template name="write-attributes"/>
        <xsl:call-template name="write-nodes">
            <xsl:with-param name="no-punct" select="true()"/>
        </xsl:call-template>
        <xsl:call-template name="write-keyword">
            <xsl:with-param name="name" select="'&gt;'"/>
        </xsl:call-template>
    </xsl:template>

    <xsl:template match="*">
        <xsl:call-template name="write-keyword">
            <xsl:with-param name="name" select="concat('&lt;', name())"/>
        </xsl:call-template>
        <xsl:call-template name="write-attributes"/>
        <xsl:call-template name="write-nodes">
            <xsl:with-param name="no-punct" select="true()"/>
        </xsl:call-template>
        <xsl:call-template name="write-keyword">
            <xsl:with-param name="name" select="'&gt;'"/>
        </xsl:call-template>
    </xsl:template>
</xsl:stylesheet>
